# Class Decimal, version 0.7.0
# Written by Eric Price <eprice@tjhsst.edu>
# Based on code written by Aahz (aahz@pobox.com)
# Currently under BSD-style license (copyright 2003)
"""
This is a stab at a decimal arithmetic module based on the decimal
arithmetic ANSI standard X3.274-1996.

Most of the information used to create this module was helpfully provided 
by Mike Cowlishaw's work at
http://www2.hursley.ibm.com/decimal/
The rest comes from Tim Peters's FixedPoint.py, particularly the string
conversion routine and general Pythonic practices.

In the current form of Decimal(), precision is finite but unbounded, with
the exception of division and square roots.

Here are some simple test cases that show how to use Decimal.  There
is a separate full-blown test harness (run Test(testdatadir).test())

>>> Decimal(0)
Decimal( (0, (0,), 0L) )
>>> Decimal("1")
Decimal( (0, (1,), 0L) )
>>> Decimal("-.0123") 
Decimal( (1, (1, 2, 3), -4L) )
>>> Decimal("1.20")
Decimal( (0, (1, 2, 0), -2L) )
>>> str(Decimal(123456789012345678901234567890L))
'123456789012345678901234567890'
>>> str(Decimal("123.45e12345678901234567890"))
'1.2345e12345678901234567892'
>>> str(Decimal("1.33") + Decimal("1.27"))
'2.60'
>>> str(Decimal("12.34") + Decimal("3.87") - Decimal("18.41"))
'-2.20'
>>> dig = Decimal(1)
>>> print dig / Decimal(3)
0.333333333
>>> dig.context.prec = 18
>>> print dig / Decimal(3)
0.333333333333333333
>>> print dig.sqrt()
1
>>> print Decimal(3).sqrt()
1.73205080756887729
>>> print Decimal(3) ** 123
4.85192780976896427E+58
>>> inf = Decimal(1) / Decimal(0)
>>> print inf
Infinity
>>> neginf = Decimal(-1) / Decimal(0)
>>> print neginf
-Infinity
>>> print neginf + inf
NaN
>>> print neginf * inf
-Infinity
>>> print dig / 0
Infinity
>>> getcontext().trap_enablers[DivisionByZero] = 1
>>> print dig / 0
Traceback (most recent call last):
  File "<stdin>", line 1, in ?
  File "/home/eprice/python/Decimal.py", line 961, in __div__
    return self._divide(other, context=context)
  File "/home/eprice/python/Decimal.py", line 1009, in _divide
    return context.raise_error(DivisionByZero, 'x / 0', sign)
  File "/home/eprice/python/Decimal.py", line 1907, in raise_error
    raise error, explanation
Decimal.DivisionByZero: x / 0
>>> c = Context()
>>> c.trap_enablers[DivisionUndefined] = 1
>>> print c.flags[DivisionUndefined]
0
>>> c.divide(Decimal(0), Decimal(0))
>>> from Decimal import *
>>> c = Context()
>>> c.trap_enablers[DivisionUndefined] = 1
>>> print c.flags[DivisionUndefined]
0
>>> print c.divide(Decimal(0), Decimal(0))
Traceback (most recent call last):
  File "<stdin>", line 1, in ?
  File "/home/eprice/python/Decimal.py", line 1987, in divide
    return a.__div__(b, context=self)
  File "/home/eprice/python/Decimal.py", line 947, in __div__
    return self._divide(other, context=context)
  File "/home/eprice/python/Decimal.py", line 978, in _divide
    return context.raise_error(DivisionUndefined, '0 / 0')
  File "/home/eprice/python/Decimal.py", line 1896, in raise_error
    raise error, explanation
Decimal.DivisionUndefined: 0 / 0
>>> print c.flags[DivisionUndefined]
1
>>> c.flags[DivisionUndefined] = 0
>>> c.trap_enablers[DivisionUndefined] = 0
>>> print c.divide(Decimal(0), Decimal(0))
NaN
>>> print c.flags[DivisionUndefined]
1
>>>
"""

# 0.7.0     2003.2.28  More changes.  Contexts done nicely, exponent limits
# 0.6.0     2003.2.24  Many changes.  No exponent limits

# Aahz:
# 0.0.8     2001.5.24  Fixed multiplication with trailing zeroes
# 0.0.7     2001.5.23  Fixed string conversion to follow spec
# 0.0.6     2001.5.20  Tim Peters's _floatToString()
# 0.0.5     2001.5.20  Now with multiplication, round(), and conversions
# 0.0.0     2001.5.18  First public release with just addition/subtraction

# To-do list:
# 
# Cleanup, hunt and kill bugs

import threading
import copy
import math
import time
from operator import xor

#Capitals:  1E+10, not 1e10

CAPITALS = 1

# Clamp:  When finishing operation, change exponents if too high:
# maxExponent: 10
# precision: 4
# clamp: 1
# add 0 1e10 -> 1.000e10

CLAMP = 0

# Default Context

BASIC_DEFAULT_CONTEXT = 'basic'

# Basic default context:
#
# flags:          all set to 0
# trap-enablers:  inexact, rounded, and subnormal are set to 0;
#                 all others are set to 1
# precision:      is set to 9
# rounding:       is set to round-half-up 

EXTENDED_DEFAULT_CONTEXT = 'extended'

# Extended default context:
# flags:          all set to 0
# trap-enablers:  all set to 0 (IEEE 854 7)
# precision:      is set to the designated single precision
# rounding:       is set to round-half-even (IEEE 854 4.1) 


DEFAULT_CONTEXT = EXTENDED_DEFAULT_CONTEXT

#Precision

BASIC_DEFAULT_PRECISION = 9
EXTENDED_DEFAULT_PRECISION = 9

DEFAULT_MAX_EXPONENT = 999999999
DEFAULT_MIN_EXPONENT = -999999999

ABSOLUTE_MAX_EXP = 999999999
ABSOLUTE_MIN_EXP = -999999999
# Power's increased prec can't go above this in test cases.


#Rounding

ROUND_DOWN = 'down'
ROUND_HALF_UP = 'half_up'
ROUND_HALF_EVEN = 'half_even'
ROUND_CEILING = 'ceiling'
ROUND_FLOOR = 'floor'
ROUND_UP = 'up'
ROUND_HALF_DOWN = 'half_down'

BASIC_DEFAULT_ROUNDING = ROUND_HALF_UP
EXTENDED_DEFAULT_ROUNDING = ROUND_HALF_EVEN

#Rounding decision

NEVER_ROUND = 'never'    # Round in division (non-divmod), sqrt ONLY
ALWAYS_ROUND = 'always'  # Every operation rounds at end.



BASIC_DEFAULT_ROUNDING_DECISION = ALWAYS_ROUND
EXTENDED_DEFAULT_ROUNDING_DECISION = ALWAYS_ROUND


#Errors

class DecimalException(ArithmeticError):
    """Base exception class, defines default things.
    
    Used exceptions derive from this.
    If an exception derives from another exception besides this (such as
    Underflow (Inexact, Rounded, Subnormal) that indicates that it is only
    called if the others are present.  This isn't actually used for
    anything, though.
    
    Attributes:
    
    default -- If the context is basic, the trap_enablers are set to
               this by default.  Extended contexts start out with them set
               to 0, regardless.

    handle  -- Called when context.raise_error is called and the
               trap_enabler is set.  First argument is self, second is the
               context.  More arguments can be given, those being after
               the explanation in raise_error (For example,
               context.raise_error(NewError, '(-x)!', self._sign) would
               call NewError().handle(context, self._sign).)

    To define a new exception, it should be sufficient to have it derive
    from DecimalException.
    """
    default = 1
    def handle(self, context, *args):
        pass


class Clamped(DecimalException):
    """Exponent of a 0 changed to fit bounds"""

class InvalidOperation(DecimalException):
    """An invalid operation was performed.
    
    Various bad things cause this:
    
    Something creates a signaling NaN
    -INF + INF
     0 * (+-)INF
     (+-)INF / (+-)INF
    x % 0
    (+-)INF % x
    x.rescale( non-integer )
    sqrt(-x) , x > 0
    0 ** 0
    x ** (non-integer)
    x ** (+-)INF
    An operand is invalid    
    """
    def handle(self, context, *args):
        if args:
            return (NaN,NaN)
        return NaN

class ConversionSyntax(InvalidOperation):
    """Trying to convert badly formed string."""
    
    def handle(self, context, *args):
        return (0, (0,), 'n') #Passed to something which uses a tuple.

class DivisionByZero(DecimalException, ZeroDivisionError):
    """Division by 0.

    x / 0
    Different from ZeroDivisionError because if the trap is disabled,
    (+-) INF is returned.
    Pass in the sign, and another argument will return a 2-tuple
    (for divmod) instead of the single number.
    """
    
    def handle(self, context, sign, double = None, *args):
        if double is not None:
            return (Infsign[sign],)*2
        return Infsign[sign]

class DivisionImpossible(InvalidOperation):
    """Cannot perform the division adequately.
    
    Result of a divide-integer or remainder option longer than precision
    Not actually raised.
    """
    
    def handle(self, context, *args):
        return (NaN, NaN)
    
class DivisionUndefined(InvalidOperation, ZeroDivisionError):
    """Undefined result of division.
    
    0 / 0
    Not a ZeroDivisionError, because it isn't infinity,
    but can be anything.
    If the trap is disabled, returns NaN.

    If arguments are passed, it returns a 2-tuple (for divmod) as opposed
    to a single tuple.
    """
    
    def handle(self, context, tup=None, *args):
        if tup is not None:
            return (NaN, NaN) #for 0 %0, 0 // 0
        return NaN

class Inexact(DecimalException):
    """Had to round, losing information."""
    default = 0

class InsufficientStorage(InvalidOperation):
    """Not enough storage space"""
    def handle(self, context, *args):
        return NaN

class InvalidContext(InvalidOperation):
    """Invalid context.  Unknown rounding, for example."""
    
    def handle(self, context, *args):
        return NaN

class LostDigits(DecimalException):
    """Actually, this is not used for the extended version."""
    pass

class Rounded(DecimalException):
    """Number got rounded (not  necessarily changed during rounding)"""
    default = 0

class Subnormal(DecimalException):
    """Exponent < Emin before rounding"""
    pass

class Overflow(Inexact, Rounded):
    """Number overflows
    
    Rescale with coefficient becoming greater than prec long
    Adjusted exponent greater than Emax
    Power's adjusted exponent above ABSOLUTE_EMAX

    Pass in the sign, and the result is:
    rounding in (half-up, half-even, half-down, up) => signed INF
    Otherwise, if the sign is negative:
        If rounding is floor, return -INF
        otherwise (rounding in (ceiling, down)) return negative number
        with highest absolute value possible (9.999999...eEmin)
    Sign positive:
        rounding = ceiling => INF
        rounding in (floor, down) => maximum finite number (9.9999...eEmax)
    """
    
    def handle(self, context, sign, *args):
        if context.rounding in (ROUND_HALF_UP, ROUND_HALF_EVEN,
                                     ROUND_HALF_DOWN, ROUND_UP):
            return Infsign[sign]
        if sign == 0:
            if context.rounding in (ROUND_CEILING,):
                return Infsign[sign]
            return Decimal((sign, (9,)*context.prec,
                            context.Emax-context.prec+1))
        if sign == 1:
            if context.rounding in (ROUND_FLOOR,):
                return Infsign[sign]
            return Decimal( (sign, (9,)*context.prec,
                             context.Emax-context.prec+1))
        

class Underflow(Inexact, Rounded, Subnormal):
    """Number too small, got rounded to 0"""
    

def filterfunct(obj):
    """Returns true if a subclass of DecimalException"""
    try:
        return issubclass(obj, DecimalException)
    except TypeError:
        return False

#ExceptionList holds the exceptions
ExceptionList = filter(filterfunct, globals().values())


def setcontext(context):
    """Set this thread's context to context."""
    threading.currentThread().__decimal_context__ = context

def getcontext():
    """Returns this thread's context.
    
    If this thread does not yet have a context, returns
    a new context and sets this thread's context.
    New contexts are copies of DefaultContext.
    """
    try:
        return threading.currentThread().__decimal_context__
    except AttributeError:
        context = Context()
        threading.currentThread().__decimal_context__ = context
        return context

class Decimal:
    """Floating point class for decimal arithmetic."""
    def __init__(self, value, context = None):
        """Initialize the Decimal class.
        
        Accepts several types of input:
        
        string
        list/tuple in form (sign, (digit_list), exponent)
        Decimal instance
        float, long, int
        """
        if context is None:
            context = getcontext()
        # String?
        # REs insist on real strings, so we can too.
        if isinstance(value, type('')): 
            if isinfinity(value):       
                self._exp = 'F'
                self._int = (0,)
                sign = isinfinity(value)
                if sign == 1:
                    self._sign = 0
                else:
                    self._sign = 1
                return
            if isnan(value):
                sig = isnan(value) - 1
                self._exp = sig and 'N' or 'n'
                self._sign = 0
                self._int = (0,) #snan or nan
                return
            self._convertString(value, context)
            # Only a string if it works here
            return
        
        # tuple/list conversion (possibly from repr())
        try:
            if len(value) != 3:
                raise ValueError
            try:
                value + (None,)
            except TypeError: 
                #Should be a similar to a list or tuple
                value + [None]   
            self._sign = value[0]
            self._int = tuple(value[1])
            self._exp = value[2]
            return
        except (TypeError, AttributeError):
            pass
        
        # Turn an intermediate value back to Decimal()
        if isinstance(value, _WorkRep):
            if value.sign == 1:
                self._sign = 0
            else:
                self._sign = 1
            self._int = tuple(value.int)
            self._exp = long(value.exp)
            return
        
        # Make a copy of Decimal
        try:
            self._sign = value._sign
            self._int = value._int
            self._exp = value._exp
            return
        except AttributeError:
            pass
        
        
        if isinstance(value, type(1)) or isinstance(value, type(1L)):
            self._convertString(str(value), context)
            return
        
        if isinstance(value, type(1.0)):
            
            
            #self._convertString(_floatToString(value), context)
            #
            # This would give a more strictly accurate representation,
            # but probably isn't what is really meant (Decimal(1.1) !=
            # Decimal('1.1') this way, but is instead Decimal(\
            # '1100000000000000088817841970012523233890533447265625e-51')
            # The extra digits really aren't worth anything, since they
            # likely weren't intended.
            
            self._convertString(str(value), context)
            return
        
        raise TypeError("Can't convert " + `value`)
    
    def _isnan(self):
        """Returns whether the number is not actually one.
        
        0 if a number
        1 if NaN
        2 if sNaN
        """
        #if self._int[0] == 'nan':
        if self._exp == 'n':
            return 1
        elif self._exp == 'N':
            return 2
        return 0
    
    def _isinfinity(self):
        """Returns whether the number is infinite
        
        0 if finite or not a number
        1 if +INF
        -1 if -INF
        """
        if self._exp == 'F':
            if self._sign:
                return -1
            return 1
        return 0
    
    def _check_nans(self, other = None, context=None):
        """Returns whether the number is not actually one.
        
        if self, other are sNaN, signal
        if self, other are NaN return nan
        return 0
        
        Done before operations.
        """
        if context is None:
            context = getcontext()
        
        if self._isnan() == 2:
            return context.raise_error(InvalidOperation, 'sNaN')
        if other is not None and other._isnan() == 2:
            return context.raise_error(InvalidOperation, 'sNaN')
        if self._isnan():
            return NaN
        if other is not None and other._isnan():
            return NaN
        return 0
    
    def _convertString(self, value, context=None):
        """Changes self's value to that in a string.
        
        A bad string causes a ConversionSyntax error.
        """
        if context is None:
            context = getcontext()
        try:
            self._sign, self._int, self._exp = _string2exact(value)
        except ValueError:
            self._sign, self._int, self._exp = \
                        context.raise_error(ConversionSyntax)
        return
    
    def __nonzero__(self):
        """Is the number non-zero?
        
        0 if self == 0
        1 if self != 0
        """
        if type(self._exp) is type(''):
            return 1
        for digit in self._int:
            if digit != 0:
                return 1
        return 0
    
    def __copy__(self, shouldround=0, context=None):
        """Create a copy of self.
        
        Also rounds, if the next argument is true.
        """
        if shouldround:
            if context is None:
                context = getcontext()
            return Decimal( (self._sign, self._int, self._exp)
                            ).fix(context=context)
        else:
            return Decimal((self._sign, self._int, self._exp))
    
    def __cmp__(self, other, context=None):
        """Compares one to another.

        -1 => a < b
        0  => a = b
        1  => a > b
        Returns Decimal instances, though.
        """
        if context is None:
            context = getcontext()
        ans = self._check_nans(other, context)
        #compare(NaN, NaN) = NaN
        if ans: return ans            
        
        if not self and not other:  
            return Decimal(0) #If both 0, sign comparison isn't certain.
        
        #If different signs, neg one is less
        if other._sign < self._sign:
            return Decimal(-1)
        if self._sign < other._sign:  
            return Decimal(1)
        
        # INF = INF
        if self._isinfinity() and other._isinfinity(): 
            return Decimal(0)
        if self._isinfinity():
            return Decimal((-1)**self._sign)
        if other._isinfinity():
            return Decimal(-((-1)**other._sign))
        
        if self.adjusted() == other.adjusted() and \
           self._int + (0,)*(self._exp - other._exp) == \
           other._int + (0,)*(other._exp - self._exp): 
            return Decimal(0) #equal, except in precision. ([0]*(-x) = [])
        elif self.adjusted() > other.adjusted() and self._int[0] != 0:
            return Decimal((-1)**self._sign)
        elif self.adjusted < other.adjusted() and other._int[0] != 0:
            return Decimal(-((-1)**self._sign))
        
        context = copy.copy(context)
        rounding = context.set_rounding(ROUND_UP) #round away from 0
        
        flags = context.ignore_all_flags()
        res = context.subtract(self, other)

        context.regard_flags(*flags)
        
        context.rounding = rounding
        
        if not res:
            return Decimal(0)
        elif res._sign:
            return Decimal(-1)
        return Decimal(1)
    
    def __hash__(self):
        """Hashes whole numbers to their long equivalent.

        Numbers equal in value hash to the same value if exp>0, only if
        they have the same precision if not.
        XXX Doesn't make too much sense-- hash(10) = hash(1e1) != hash(100e-1)
        """
        if self._exp >= 0:
            return hash(long(self))
        else:
            return hash( (self._sign, self._int, self._exp) )

    def __repr__(self):
        """Represents the number as an instance of Decimal.
        
        Not pretty, but eval(repr(self)) == self
        """
        return "Decimal( (" + `self._sign` + ", " +  `self._int` + \
               ", " + `self._exp` + ") )"

    def __str__(self, eng = 0, context=None):
        """Return the string representation of the number.
        
        If eng is set, it is written in engineering notation
        (in which the exponent is a multiple of 3).  Otherwise
        scientific notation is used.
        
        see http://www2.hursley.ibm.com/decimal/daconvs.html for a complete
        specification of the conversion.
        """
        if self._isnan():
            if self._isnan() == 2:
                return 'sNaN'
            return 'NaN'
        if self._isinfinity():
            sign = self._isinfinity()
            if sign > 0:
                return 'Infinity'
            return '-Infinity'

        if context is None:
            context = getcontext()

        tmp = map(str, self._int)
        numdigits = len(self._int)
        leftdigits = self._exp + numdigits
        if eng:
            dotplace = (leftdigits-1)%3+1
            adjexp = leftdigits -1 - (leftdigits-1)%3
        else:
            adjexp = leftdigits-1
            dotplace = 1
        if self._exp == 0:
            pass
        elif self._exp < 0 and adjexp >= 0:
            tmp.insert(leftdigits, '.')
        elif self._exp < 0 and adjexp >= -6:
            tmp[0:0] = ['0'] * int(-leftdigits)
            tmp.insert(0, '0.')
        else:
            if numdigits > dotplace:
                tmp.insert(dotplace, '.')
            elif numdigits < dotplace:
                tmp.extend(['0']*(dotplace-numdigits))
            if adjexp:
                if not context.capitals:
                    tmp.append('e')
                else:
                    tmp.append('E')
                    if adjexp > 0:
                        tmp.append('+')
                tmp.append(str(adjexp))
        if eng:
            while tmp[0:1] == ['0']:
                tmp[0:1] = []
            if len(tmp) == 0 or tmp[0] == '.' or tmp[0].lower() == 'e':
                tmp[0:0] = ['0']
        if self._sign:
            tmp.insert(0, '-')
                
        return ''.join(tmp)

    def sci(self, context=None):
        """Round if necessary, first. then str(self)"""
        if context is None:
            context = getcontext()
        if context.rounding_decision in (ALWAYS_ROUND,):
            return str(self.fix(context=context))
        return str(self)
    
    def __neg__(self, context=None):
        """Returns a copy with the sign switched.
        
        Rounds, if it has reason.
        """
        if context is None:
            context = getcontext()
        ans = self._check_nans(context=context)
        if ans: return ans


        if not self:
            # -Decimal('0') is Decimal('0'), not Decimal('-0')
            sign = 0
        elif self._sign:
            sign = 0
        else:
            sign = 1
        if context.rounding_decision in (ALWAYS_ROUND,):
            return Decimal((sign, self._int, self._exp)).fix(context=context)
        return Decimal( (sign, self._int, self._exp))
    def __pos__(self, context=None):
        """Returns a copy, unless it is a sNaN.

        Rounds the number (if more then precision digits)
        """
        if context is None:
            context = getcontext()
        ans = self._check_nans(context=context)
        if ans: return ans

        sign = self._sign
        if not self:
            # + (-0) = 0
            sign = 0
        
        if context.rounding_decision in (ALWAYS_ROUND,):
            ans = self.fix(context=context)
        else:
            ans = self.__copy__()
        ans._sign = sign
        return ans

    def __abs__(self, round=1, context=None):
        """Returns the absolute value of self.

        If the second argument is 0, do not round.
        """
        if context is None:
            context = getcontext()
        ans = self._check_nans(context=context)
        if ans: return ans
        if not round:
            context = copy.copy(context)
            context.set_rounding_decision(NEVER_ROUND)

        if self._sign:
            ans = context.minus(self)
        else:
            ans = context.plus(self)

        return ans


    def __add__(self, other, context=None):
        """Returns self + other.

        -INF + INF (or the reverse) cause InvalidOperation errors.
        """
        if context is None:
            context = getcontext()
        
        ans = self._check_nans(other, context)
        if ans: return ans


        if self._isinfinity():
            #If both INF, same sign => same as both, opposite => error.
            if self._sign != other._sign and other._isinfinity():
                return context.raise_error(InvalidOperation, '-INF + INF')
            return self.__copy__() 
        if other._isinfinity():
            return other.__copy__()  #Can't both be infinity here

        shouldround = context.rounding_decision in (ALWAYS_ROUND,)

        exp = min(self._exp, other._exp)
        negativezero = 0
        if context.rounding == ROUND_FLOOR and self._sign != other._sign:
            #If the answer is 0, the sign should be negative, in this case.
            negativezero = 1

        if not self and not other:
            sign = min(self._sign, other._sign)
            if negativezero:
                sign = 1
            return Decimal( (sign, (0,), exp))
        if not self:
            if exp < other._exp - context.prec-1:
                exp = other._exp - context.prec-1
            ans = other.rescale(Decimal(exp), watchexp=0, context=context)
            if shouldround:
                ans = ans.fix(context=context)
            return ans
        if not other:
            if exp < self._exp - context.prec-1:
                exp = self._exp - context.prec-1
            ans = self.rescale(Decimal(exp), watchexp=0, context=context)
            if shouldround:
                ans = ans.fix(context=context)
            return ans

        op1 = _WorkRep(self)
        op2 = _WorkRep(other)
        op1, op2 = _normalize(op1, op2, shouldround, context.prec)
        
        result = _WorkRep()
        
        if op1.sign != op2.sign:
            diff = cmp(abs(op1), abs(op2))
            # Equal and opposite
            if diff == 0: 
                if exp < context.Etiny():
                    exp = context.Etiny()
                    context.raise_error(Clamped)
                return Decimal((negativezero, (0,), exp))
            if diff < 0:
                op1, op2 = op2, op1  
                #OK, now abs(op1) > abs(op2)
            if op1.sign == -1:
                result.sign = -1
                op1.sign, op2.sign = op2.sign, op1.sign
            else:
                result.sign = 1  
                #So we know the sign, and op1 > 0.
        elif op1.sign == -1:
            result.sign = -1
            op1.sign, op2.sign = (1, 1)
        else:
            result.sign = 1  
        #Now, op1 > abs(op2) > 0

        carry, loan = 0, 0
        op1.int.reverse()
        op2.int.reverse()
        result.int = [0]*len(op1.int)
        for i in range(len(op1.int)):
            tmp = op1.int[i] + (op2.sign * op2.int[i]) + carry - loan
            carry, loan = (0, 0)
            if tmp > 9:
                carry = 1
                tmp = tmp - 10
            if tmp < 0:
                loan = 1
                tmp = tmp + 10
            result.int[i] = tmp
        if carry:
            result.int.append(1)
        if loan:
            raise "What are we doing here?"
        while result.int[-1] == 0:
            result.int.pop()
        result.int.reverse()
        result.exp = op1.exp
        ans = Decimal(result)
        if shouldround:
            ans = ans.fix(context=context)
        return ans

    __radd__ = __add__

    def __sub__(self, other, context=None):
        """Return self + (-other)"""
        if context is None:
            context = getcontext()
        # -Decimal(0) = Decimal(0), which we don't want
        # (-0 - 0 = -0 + (-0) = -0, but -0 + 0 = 0.)
        # so we change the sign directly.
        tmp = other.__copy__()
        tmp._sign = 1-tmp._sign 

        return context.add(self, tmp) 
                                   

    def __rsub__(self, other, context=None):
        if context is None:
            context = getcontext()
        tmp = self.__copy__()
        tmp._sign = 1 - tmp._sign
        return context.add(other, tmp)
    
    def increment(self, round=1, context=None):
        """Special case of add, adding 1eExponent

        
        Since it is common, (rounding, for example) this adds
        (sign)*one E self._exp to the number more efficiently than add.

        For example:
        Decimal('5.624e10').increment() == Decimal('5.625e10')
        """
        if context is None:
            context = getcontext()
        ans = self._check_nans(context=context)
        if ans: return ans
        
        L = list(self._int)
        L[-1] += 1
        spot = len(L)-1
        while L[spot] == 10:
            L[spot] = 0
            if spot == 0:
                L[0:0] = [1]
                break
            L[spot-1] += 1
            spot -= 1
        ans = Decimal((self._sign, L, self._exp))

        if round and context.rounding_decision in (ALWAYS_ROUND,):
            ans = ans.fix(context=context)
        return ans
        
    def __mul__(self, other, context=None):
        """Return self * other.

        (+-) INF * 0 (or its reverse) raise InvalidOperation.
        """
        if context is None:
            context = getcontext()
        ans = self._check_nans(other, context)
        if ans: return ans
        
        resultsign = xor(self._sign, other._sign)
        if self._isinfinity():
            if not other:
                return context.raise_error(InvalidOperation, '(+-)INF * 0')
            return Infsign[resultsign]
        
        if other._isinfinity():
            if not self:
                return context.raise_error(InvalidOperation, '0 * (+-)INF')
            return Infsign[resultsign]
        
        
        resultexp = self._exp + other._exp
        shouldround = context.rounding_decision in (ALWAYS_ROUND,)
        
        # Special case for multiplying by zero
        if not self or not other:
            ans = Decimal((resultsign, (0,), resultexp))
            if shouldround:
                #Fixing in case the exponent is out of bounds
                ans = ans.fix(context=context)
            return ans

        # Special case for multiplying by power of 10
        if self._int == (1,):
            ans = Decimal((resultsign, other._int, resultexp))
            if shouldround:
                ans = ans.fix(context=context)
            return ans
        if other._int == (1,):
            ans = Decimal((resultsign, self._int, resultexp))
            if shouldround:
                ans = ans.fix(context=context)
            return ans
        
        op1 = list(self._int)
        op2 = list(other._int)
        op1.reverse()
        op2.reverse()
        # Minimize Decimal additions
        if len(op2) > len(op1):
            op1, op2 = op2, op1

        accumulator = [0]*(len(self._int) + len(other._int))
        for i in range(len(op2)):
            if op2[i] == 0:
                continue
            mult = op2[i]
            carry = 0
            for j in range(len(op1)):
                carry, accumulator[i+j] = divmod( mult * op1[j] + carry
                                                  + accumulator[i+j], 10)
                
            if carry:
                accumulator[i + j + 1] += carry
        while not accumulator[-1]:
            accumulator.pop()
        accumulator.reverse()
        
        ans = Decimal( (resultsign, accumulator, resultexp))
        if shouldround:
            ans = ans.fix(context=context)
        
        return ans
    __rmul__ = __mul__

    def __div__(self, other, context=None):
        """Return self / other."""
        return self._divide(other, context=context)
    __truediv__ = __div__
    

    def _divide(self, other, divmod = 0, context=None):
        """Return a / b, to context.prec precision.

        divmod:
        0 => true division
        1 => (a //b, a%b)
        2 => a //b
        3 => a%b

        Actually, if divmod is 2 or 3 a tuple is returned, but errors for
        computing the other value are not raised.
        """
        if context is None:
            context = getcontext()

        ans = self._check_nans(other, context)
        if ans:
            if divmod:
                return (ans, ans)
            else:
                return ans


        sign = xor(self._sign, other._sign)
        if not self and not other:
            if divmod:
                return context.raise_error(DivisionUndefined, '0 / 0', 1)
            return context.raise_error(DivisionUndefined, '0 / 0')
        if self._isinfinity() and other._isinfinity():
            if not divmod:
                return context.raise_error(InvalidOperation, '(+-)INF/(+-)INF')
            else:
                return (context.raise_error(InvalidOperation,
                                         '(+-)INF // (+-)INF'),
                        context.raise_error(InvalidOperation,
                                         '(+-)INF % (+-)INF'))

        if not divmod:
            if other._isinfinity():
                return Decimal((sign, (0,), 0))
            #These two have different precision.
            if not self:               
                exp = self._exp - (len(other._int)-1 +other._exp)
                if exp < context.Etiny():
                    exp = context.Etiny()
                    context.raise_error(Clamped, '0e-x / y') 
                if exp > context.Emax:
                    exp = context.Emax
                    context.raise_error(Clamped, '0e+x / y') 
                return Decimal( (sign, (0,), exp) )
                                 
            if not other:
                return context.raise_error(DivisionByZero, 'x / 0', sign)
            if self._isinfinity():
                return Infsign[sign]
        if divmod:
            if other._isinfinity():
                return (Decimal((sign, (0,), 0)),
                        self.__copy__())
            if not self:
                otherside = self.__copy__()
                otherside._exp = min(self._exp, other._exp)
                return (Decimal((sign, (0,), 0)),  otherside)
                        
            if not other:
                return context.raise_error(DivisionByZero, 'divmod(x,0)',
                                           sign, 1)
            if self._isinfinity():
                if divmod == 1:
                    return (Infsign[sign],
                            context.raise_error(InvalidOperation, 'INF % x'))
                elif divmod == 2:
                    return (Infsign[sign], NaN)
                elif divmod == 3:
                    return (Infsign[sign],
                            context.raise_error(InvalidOperation, 'INF % x'))
                
                     
        #OK, so neither = 0, INF

        shouldround = context.rounding_decision in (ALWAYS_ROUND,)

        #If we're dividing into ints, and self < other, stop.
        #self.__abs__(0) does not round.
        if divmod and (self.__abs__(0, context) < other.__abs__(0, context)):
            
            if divmod == 1 or divmod == 3:
                ans2 = self.__copy__()
                if shouldround:
                    ans2 = ans2.fix(context=context)
                return (Decimal( (sign, (0,), 0) ),
                        ans2)
            
            elif divmod == 2:
                #Don't round the mod part, if we don't need it.
                return (Decimal( (sign, (0,), 0) ), self.__copy__())
        
        if sign:
            sign = -1
        else:
            sign = 1
        adjust = 0
        op1 = _WorkRep(self)
        op2 = _WorkRep(other)
        op1, op2, adjust = _adjust_coefficients(op1, op2)
        res = _WorkRep( (sign, [0], (op1.exp - op2.exp)) )
        if divmod and res.exp > context.prec + 1:
            return context.raise_error(DivisionImpossible)
            
        ans = None
        while 1:
            while( (len(op2.int) < len(op1.int) and op1.int[0]) or
                   (len(op2.int) == len(op1.int) and op2.int <= op1.int)):
                   #Meaning, while op2.int < op1.int, when normalized.
                res._increment()
                op1._subtract(op2.int)
            if res.exp == 0 and divmod:
                if len(res.int) > context.prec and shouldround:
                    return context.raise_error(DivisionImpossible)
                otherside = Decimal(op1)
                frozen = context.ignore_all_flags()
                
                exp = min(self._exp, other._exp)
                otherside = otherside.rescale(Decimal(exp), context=context,
                                              watchexp=0)
                context.regard_flags(*frozen)
                if not otherside:
                    otherside._exp = 0
                if shouldround:
                    otherside = otherside.fix(context=context)
                return (Decimal(res), otherside)

            if op1.int == [0]*len(op1.int) and adjust >= 0 and not divmod:
                break
            if (len(res.int) > context.prec) and shouldround:
                if divmod:
                    return context.raise_error(DivisionImpossible)
                shouldround=1
                # Really, the answer is a bit higher, so adding a one to
                # the end will make sure the rounding is right.
                if op1.int != [0]*len(op1.int):  
                    res.int.append(1)            
                    res.exp -= 1                 
                                                 
                break
            res.exp -= 1
            adjust += 1
            res.int.append(0)
            op1.int.append(0)
            op1.exp -= 1

            if res.exp == 0 and divmod and (len(op2.int) > len(op1.int) or
                                            (len(op2.int) == len(op1.int) and
                                             op2.int > op1.int)):
                #Solves an error in precision.  Same as a previous block.

                if len(res.int) > context.prec and shouldround:
                    return context.raise_error(DivisionImpossible)
                otherside = Decimal(op1)
                frozen = context.ignore_all_flags()

                exp = min(self._exp, other._exp)
                otherside = otherside.rescale(Decimal(exp), context=context)

                context.regard_flags(*frozen)
                if not otherside:
                    otherside._exp = 0

                return (Decimal(res), otherside)

        ans = Decimal(res)
        if shouldround:
            ans = ans.fix(context=context)
        return ans

    def __rdiv__(self, other):
        raise NotImplementedError        

    def __divmod__(self, other, context=None):
        """
        (self // other, self % other)
        """
        return self._divide(other, 1, context)

    def __rdivmod__(self, other):
        raise NotImplementedError

    def __mod__(self, other, context=None):
        """
        self % other
        """
        if context is None:
            context = getcontext()
        ans = self._check_nans(other, context)
        if ans: return ans

        if self and not other:
            return context.raise_error(InvalidOperation, 'x % 0')

        return self._divide(other, 3, context)[1]

    def __rmod__(self, other):
        raise NotImplementedError

    def remainder_near(self, other, context=None):
        """
        Remainder nearest to 0-  abs(remainder-near) <= other/2
        """
        if context is None:
            context = getcontext()
        ans = self._check_nans(other, context)
        if ans: return ans
        if self and not other:
            return context.raise_error(InvalidOperation, 'x % 0')

        # If DivisionImpossible causes an error, do not leave Rounded/Inexact
        # ignored in the calling function.
        context = copy.copy(context)
        flags = context.ignore_flags(Rounded, Inexact)
        #keep DivisionImpossible flags
        (side, r) = context.divmod(self, other)
        
        if r._isnan():
            context.regard_flags(*flags)
            return r

        context = copy.copy(context)
        rounding = context.set_rounding_decision(NEVER_ROUND)

        if other._sign:
            comparison = context.divide(other, Decimal(-2))
        else:
            comparison = context.divide(other, Decimal(2))


        context.set_rounding_decision(rounding)
        context.regard_flags(*flags)

        s1, s2 = r._sign, comparison._sign
        r._sign, comparison._sign = 0, 0

        if r < comparison:
            r._sign, comparison._sign = s1, s2
            #Get flags now
            context.divmod(self, other) 
            return r.fix(context=context) 
        r._sign, comparison._sign = s1, s2

        
        rounding = context.set_rounding_decision(NEVER_ROUND)

        (side, r) = context.divmod(self, other)
        context.set_rounding_decision(rounding)
        if r._isnan():
            return r


        decrease = not side._iseven()
        rounding = context.set_rounding_decision(NEVER_ROUND)
        side = context.abs(side)
        context.set_rounding_decision(rounding)

        s1, s2 = r._sign, comparison._sign
        r._sign, comparison._sign = 0, 0
        if r > comparison or decrease and r == comparison:
            r._sign, comparison._sign = s1, s2
            context.prec += 1
            if len((context.add(side,Decimal(1))._int)) >= context.prec:
                context.prec -= 1
                return context.raise_error(DivisionImpossible)[1]
            context.prec -= 1
            if self._sign == other._sign:
                r = context.subtract(r, other)
            else:
                r = context.add(r, other)
        else:
            r._sign, comparison._sign = s1, s2
                
        return r.fix(context=context)
    
    def __floordiv__(self, other, context=None):
        """self // other"""
        return self._divide(other, 2, context)[0]
    
    def __float__(self):
        """Float representation."""
        return float(str(self))
    
    def __long__(self):
        """Converts self to a long, truncating if necessary."""
        if not self:
            return 0L
        sign = '-'*self._sign
        if self._exp >= 0:
            s = sign + ''.join(map(str, self._int)) + '0'*self._exp
            return long(s)
        s = sign + ''.join(map(str, self._int))[:self._exp]
        return long(s)
        tmp = list(self._int)
        tmp.reverse()
        val = 0L
        while tmp:
            val *= 10
            val += tmp.pop()
        return long(((-1) ** self._sign) * val * 10.**int(self._exp))
    
    def __int__(self):
        """Converts to a regular int.

        Equivalent to int(long(self))
        """
        return int(self.__long__())
    
    def fix(self, prec=None, rounding=None, folddown=None, context=None):
        """Round if it is necessary to keep self within prec precision.

        Rounds and fixes the exponent.  Does not raise on a sNaN.
        
        Arguments:
        self - Decimal instance
        prec - precision to which  to round.  By default, the context decides.
        rounding - Rounding method.  By default, the context decides.
        folddown - Fold down high elements, by default context.clamp
        context - context used.
        """
        if self._isinfinity() or self._isnan():
            return self
        if context is None:
            context = getcontext()
        if prec is None:
            prec = context.prec
        ans = self.__copy__()
        ans = ans.fixexponents(prec, rounding, folddown=folddown, context=context)
        if len(ans._int) > prec:
            ans = ans.round(prec, rounding, context=context)
        ans = ans.fixexponents(prec, rounding, folddown=folddown, context=context)
        return ans
    def fixexponents(self, prec=None, rounding=None, folddown=None, context=None):
        """Fix the exponents and return a copy with the exponent in bounds."""
        if self._isinfinity():
            return self
        if context is None:
            context = getcontext()
        if prec is None:
            prec = context.prec
        if folddown is None:
            folddown = context.clamp
        Emin, Emax = context.Emin, context.Emax
        Etop = context.Etop()
        ans = self.__copy__()
        if ans.adjusted() < Emin:
            Etiny = context.Etiny()
            if ans._exp < Etiny:
                if not ans:
                    ans._exp = Etiny
                    context.raise_error(Clamped)
                    return ans
                ans = ans.rescale(Decimal(Etiny), context=context)
                #It isn't zero, and exp < Emin => subnormal
                context.raise_error(Subnormal)
                if context.flags[Inexact]:
                    context.raise_error(Underflow)
            else:
                if ans:
                    #Only raise subnormal if non-zero.
                    context.raise_error(Subnormal)
        elif folddown and ans._exp > Etop:
            context.raise_error(Clamped)
            ans = ans.rescale(Decimal(Etop), context=context)
        elif ans.adjusted() > Emax:
            if not ans:
                ans._exp = Emax
                context.raise_error(Clamped)
                return ans
            context.raise_error(Inexact)
            context.raise_error(Rounded)
            return context.raise_error(Overflow, 'above Emax', ans._sign)
        return ans
    def round(self, prec=None, rounding=None, context=None):
        """Returns a rounded version of self.

        You can specify the precision or rounding method.  Otherwise, the
        context determines it.
        """

        if context is None:
            context = getcontext()
        ans = self._check_nans(context=context)
        if ans: return ans

        if self._isinfinity():
            return self.__copy__()

        if rounding is None:
            rounding = context.rounding
        if prec is None:
            prec = context.prec

        if not self:
            if prec <= 0:
                dig = (0,)
                exp = len(self._int) - prec + self._exp
            else:
                dig = (0,) * prec 
                exp = len(self._int) + self._exp - prec
            ans = Decimal((self._sign, dig, exp))
            context.raise_error(Rounded)
            return ans

        if prec == 0:
            temp = self.__copy__()
            temp._int = (0,)+temp._int
            prec = 1
        elif prec < 0:
            exp = self._exp + len(self._int) - prec - 1
            temp = Decimal( (self._sign, (0, 1), exp))
            prec = 1
        else:
            temp = self.__copy__()

        numdigits = len(temp._int)
        if prec == numdigits:
            return temp

        # See if we need to extend precision
        expdiff = prec - numdigits
        if expdiff > 0:
            tmp = list(temp._int)
            tmp.extend([0] * expdiff)
            ans =  Decimal( (temp._sign, tmp, temp._exp - expdiff))
            return ans

        #OK, but maybe all the lost digits are 0.
        all0 = 1
        for digit in self._int[expdiff:]:
            if digit != 0:
                all0 = 0
                break
        if all0:
            ans = Decimal( (temp._sign, temp._int[:prec], temp._exp - expdiff))
            #Rounded, but not Inexact
            context.raise_error(Rounded)  
            return ans

        # Okay, let's round and lose data
        # get rounding method function:
        rounding_functions = filter(lambda name: name[:6] == '_round',
                                    self.__class__.__dict__.keys())
        def value(name):
            """Find the global value associated with the name of a function."""
            return globals()[name[1:].upper()]

        this_function = filter(lambda name, value=value, rounding=rounding:
                               value(name) == rounding, rounding_functions)[0]
        this_function = getattr(temp, this_function)
        #Now we've got the rounding function

        if prec != context.prec:
            context = copy.copy(context)
            context.prec = prec
        ans = this_function(prec, expdiff, context)
        context.raise_error(Rounded)
        context.raise_error(Inexact, 'Changed in rounding')
        
        return ans
        
    def _round_down(self, prec, expdiff, context):
        """Also known as round-towards-0, truncate."""
        return Decimal( (self._sign, self._int[:prec], self._exp - expdiff) )


    def _round_half_up(self, prec, expdiff, context, tmp = None):
        """Rounds 5 up (away from 0)"""

        if tmp is None:
            tmp = Decimal( (self._sign,self._int[:prec], self._exp - expdiff))
        if self._int[prec] >= 5:
            tmp = tmp.increment(round=0, context=context)
            if len(tmp._int) > prec:
                return Decimal( (tmp._sign, tmp._int[:-1], tmp._exp + 1))
        return tmp

    def _round_half_even(self, prec, expdiff, context):
        """Round 5 to even, rest to nearest."""

        tmp = Decimal( (self._sign, self._int[:prec], self._exp - expdiff))
        half = (self._int[prec] == 5)
        if half:
            for digit in self._int[prec+1:]:
                if digit != 0:
                    half = 0
                    break
        if half:
            if self._int[prec-1] %2 == 0:
                return tmp
        return self._round_half_up(prec, expdiff, context, tmp)

    def _round_half_down(self, prec, expdiff, context):
        """Round 5 down"""

        tmp = Decimal( (self._sign, self._int[:prec], self._exp - expdiff))
        half = (self._int[prec] == 5)
        if half:
            for digit in self._int[prec+1:]:
                if digit != 0:
                    half = 0
                    break
        if half:
            return tmp
        return self._round_half_up(prec, expdiff, context, tmp)

    def _round_up(self, prec, expdiff, context):
        """Rounds away from 0."""
        tmp = Decimal( (self._sign, self._int[:prec], self._exp - expdiff) )
        for digit in self._int[prec:]:
            if digit != 0:
                tmp = tmp.increment(round=1, context=context)
                if len(tmp._int) > prec:
                    return Decimal( (tmp._sign, tmp._int[:-1], tmp._exp + 1))
                else:
                    return tmp
        return tmp
    def _round_ceiling(self, prec, expdiff, context):
        """Rounds up (not away from 0 if negative.)"""
        if self._sign:
            return self._round_down(prec, expdiff, context)
        else:
            return self._round_up(prec, expdiff, context)

    def _round_floor(self, prec, expdiff, context):
        """Rounds down (not towards 0 if negative)"""
        if not self._sign:
            return self._round_down(prec, expdiff, context)
        else:
            return self._round_up(prec, expdiff, context)

    def fixedPoint(self, digits, rounding = None, context=None):
        """Rounds to a number of digits around the decimal point.

        Convenience function to allow rounding to a specified number of
        places after the decimal point.  Negative numbers indicate
        rounding before the decimal point.

        >>> str(Decimal("1234.34567").fixedPoint(2))
        '1234.35'
        """
        numdigits = len(self._int)+self._exp
        ans = self.round(numdigits+digits, rounding, context=context)
        return ans

    def trim(self, context=None):
        """Removes 0s on the right, makes any 0 0e0"""
        if self._isinfinity() or self._isnan():
            return self
        if context is None:
            context = getcontext()
        #self._check_nans(context=context)
        tmp = self.__copy__()
        if not tmp:
            tmp._int = (0,)
            tmp._exp = 0
            return tmp

        if tmp._int[-1] != 0 or tmp._exp == 0:
            # No change necessary.
            return tmp
        shift = -1
        while ((self._exp - shift <= 0 or self._exp > 0) and
               tmp._int[shift] == 0):
            shift -= 1
        tmp._int = tmp._int[:shift+1]
        tmp._exp -= (shift + 1)
        return tmp
    

    def eng(self, context=None):
        """Convert to engineering-type string.

        Engineering notation has an exponent which is a multiple of 3, so there
        are up to 3 digits left of the decimal place.

        Same rules for when in exponential and when as a value as in __str__.
        """
        if context is None:
            context = getcontext()
        shouldround = context.rounding_decision in (ALWAYS_ROUND,)
        ans = self
        if shouldround:
            ans = ans.fix(context=context)
        return ans.__str__(1)

    def __pow__(self, n, modulo = None, context=None):
        """Return self ** n (mod modulo)

        If modulo is None (default), don't take it mod modulo.
        """
        if context is None:
            context = getcontext()

        if n._isinfinity() or n.adjusted() > 8:
            return context.raise_error(InvalidOperation, 'x ** INF')

        ans = self._check_nans(n, context)
        if ans: return ans

        if not n._isinfinity() and not n.isinteger():
            return context.raise_error(InvalidOperation, 'x ** (non-integer)')
        
        
        if not self and not n:
            return context.raise_error(InvalidOperation, '0 ** 0')

        if not n:
            return Decimal(1)

        if self == Decimal(1):
            return Decimal(1)

        sign = self._sign and not n._iseven()
        n = long(n)

        if self._isinfinity():
            if modulo:
                return context.raise_error(InvalidOperation, 'INF % x')
            if n > 0:
                return Infsign[sign]
            return Decimal( (sign, (0,), 0) )

        if not modulo and n > 0 and (self._exp + len(self._int)
                                     - 1) * n > context.Emax:
            tmp = self.__copy__()
            tmp._sign = sign
            context.raise_error(Rounded)
            context.raise_error(Inexact)
            context.raise_error(Overflow, 'Big power', sign)
        
        elength = len(str(abs(n)))
        firstprec = context.prec

        if firstprec + elength + 1 > ABSOLUTE_MAX_EXP:
            return context.raise_error(Overflow, 'Too much precision.', sign)
        
        mul = self.__copy__()
        val = Decimal(1)
        context = copy.copy(context)
        context.prec = firstprec + elength + 1
        rounding = context.rounding
        if n < 0:
            #n is a long now, not Decimal instance
            n = -n 
            mul = context.divide(Decimal(1), mul)

        shouldround = context.rounding_decision in (ALWAYS_ROUND,)

        spot = 1
        while spot <= n:
            spot <<= 1

        spot >>= 1 
        #Spot is the highest power of 2 less than n
        while spot:
            val = context.multiply(val, val)
            if val._isinfinity():
                val = Infsign[sign]
                break
            if spot & n:
                val = context.multiply(val, mul)
            if modulo is not None:
                val = context.remainder(val, modulo)
            spot >>= 1
        context.prec = firstprec
        if shouldround:
            return val.fix(context=context)
        return val


    def normalize(self, context=None):
        """Normalize- strip trailing 0s, change anything equal to 0 to 0e0"""
        if context is None:
            context = getcontext()
        
        ans = self._check_nans(context=context)
        if ans: return ans
        
        copy = self.fix(context=context)
        if copy._isinfinity():
            return copy
        

        if not copy:
            return Decimal( (copy._sign, (0,), 0) )
        end = len(copy._int)
        exp = copy._exp
        while copy._int[end-1] == 0:
            exp += 1
            end -= 1
        return Decimal( (copy._sign, copy._int[:end], exp) )

    def rescale(self, exp, rounding = None, context=None, watchexp = 1):
        """Rescales so that the exponent is exp.

        exp = exp to scale to
        rounding = rounding version
        watchexp: if set (default) an error is returned if exp is greater
        than Emax or less than Etiny.
        """
        if context is None:
            context = getcontext()

        ans = self._check_nans(exp, context)
        if ans:
            if exp._isinfinity():
                return context.raise_error(InvalidOperation, 'rescale(a, INF)')
            return ans


        out = 0
        if self._isinfinity() and not exp._isinfinity():
            #Even if exp not in bounds
            return self.__copy__()

        if not exp.isinteger():
            return context.raise_error(InvalidOperation, 'rescale(a, non-int)')

        if exp._isinfinity():
            return context.raise_error(InvalidOperation, 'rescale(a, INF)')
        exp = long(exp)
        if watchexp and (context.Emax < exp or context.Etiny() > exp):
            return context.raise_error(InvalidOperation, 'rescale(a, INF)')



        if not self:
            ans = self.__copy__()
            ans._int = (0,)
            ans._exp = exp
            return ans
        
        diff = self._exp - exp
        digits = len(self._int)+diff
        
        if watchexp and digits > context.prec:
            context.raise_error(Inexact)
            context.raise_error(Rounded)
            return context.raise_error(Overflow, 'Rescale > prec', self._sign)
        
        tmp = self.__copy__()
        tmp._int = (0,)+tmp._int
        digits += 1

        prevexact = context.flags[Inexact]
        if digits < 0:
            tmp._exp = -digits + tmp._exp 
            tmp._int = (0,1)
            digits = 1
        tmp = tmp.round(digits, rounding, context=context)

        if tmp._int[0] == 0 and len(tmp._int) > 1:
            tmp._int = tmp._int[1:]
        tmp._exp = exp


        if tmp and tmp.adjusted() < context.Emin:
            context.raise_error(Subnormal)
            if context.flags[Inexact] or prevexact:
                context.raise_error(Underflow)
        return tmp
            
    def round_to_integer(self, rounding = None, context=None):
        """Rounds to the nearest integer."""
        return self.rescale(Decimal(0), rounding, context=context)

    def sqrt(self, context=None):
        """Return the sqrt of self.
        
        Uses a converging algorithm (Xn+1 = 0.5*(Xn + self / Xn))
        Should quadratically approach the right answer.
        """
        if context is None:
            context = getcontext()
        
        ans = self._check_nans(context=context)
        if ans: return ans

        if not self:
            if self._sign == 1:
                #sqrt(-0) = -0
                return Decimal( (1, (0,), 0))
            else:
                return Decimal( (0, (0,), 0))

        if self._sign == 1:
            return context.raise_error(InvalidOperation, 'sqrt(-x), x > 0')

        if self._isinfinity():
            return self.__copy__()

        tmp = self.__copy__()

        expadd = tmp._exp / 2
        if tmp._exp % 2 == 1:
            tmp._int += (0,)
            tmp._exp = 0
        else:
            tmp._exp = 0

        context = copy.copy(context)
        flags = context.ignore_all_flags()
        firstprec = context.prec
        context.prec = 3
        if tmp.adjusted() % 2 == 0:
            ans = Decimal( (0, (8,1,9), tmp.adjusted()  - 2) )
            ans = context.add(ans, context.multiply(Decimal((0, (2,5,9), -2)),
                                                    tmp))
            ans._exp -= 1 + tmp.adjusted()/2
        else:
            ans = Decimal( (0, (2,5,9), tmp._exp + len(tmp._int)- 3) )
            ans = context.add(ans, context.multiply(Decimal((0, (8,1,9), -3)),
                                                    tmp))
            ans._exp -= 1 + tmp.adjusted()/2

        #ans is now a linear approximation.

        
        Emax, Emin = context.Emax, context.Emin
        context.Emax, context.Emin = ABSOLUTE_MAX_EXP, ABSOLUTE_MIN_EXP

        half = Decimal('0.5')

        count = 1
        maxp = firstprec + 2
        rounding = context.set_rounding(ROUND_HALF_EVEN)
        while 1:
            context.prec = min(2*context.prec - 2, maxp)
            ans = context.multiply(half, context.add(ans, \
                                                context.divide(tmp, ans)))
            if context.prec == maxp:
                break

        #round to the answer's precision-- the only error can be 1 ulp.
        context.prec = firstprec
        prevexp = ans.adjusted()
        ans = context.round(ans)

        #Now, check if the other last digits are better.
        context.prec = firstprec + 1
        # In case we rounded up another digit and we should actually go lower.
        if prevexp != ans.adjusted():  
            ans._int += (0,)           
            ans._exp -= 1

        lower = context.subtract(ans, Decimal((0, (5,), ans._exp-1) ))
        context.set_rounding(ROUND_UP)
        if (context.multiply(lower, lower))> (tmp):
            ans = context.subtract(ans, Decimal( (0, (1,), ans._exp) ))
        else:
            upper = context.add(ans, Decimal((0, (5,), ans._exp-1) ))
            context.set_rounding(ROUND_DOWN)
            if context.multiply(upper, upper) < tmp:
                ans = context.add(ans, Decimal( (0, (1,), ans._exp) ))
            
                
        ans._exp += expadd
        
        context.prec = firstprec
        context.rounding = rounding
        ans = ans.fix(context=context).normalize(context=context)

        rounding = context.set_rounding_decision(NEVER_ROUND)
        if not context.multiply(ans, ans) == self:
            # Only rounded/inexact if here.
            context.regard_flags(flags)  
            context.raise_error(Rounded) 
            context.raise_error(Inexact)
        else:
            context.regard_flags(flags)
        context.Emax, context.Emin = Emax, Emin
        if ans._exp*2 != self._exp:
            # Must have rounded somewhere.
            context.raise_error(Rounded)
        return ans.normalize(context=context)

    def max(self, other, context=None):
        """Returns the larger value.
        
        like max(self, other) except if one is not a number, returns
        NaN (and signals if one is sNaN).  Also rounds.
        """
        if context is None:
            context = getcontext()

        ans = self._check_nans(other, context)
        if ans: return ans

        ans = self
        if self < other:
            ans = other
        shouldround = context.rounding_decision in (ALWAYS_ROUND,)
        if shouldround:
            ans = ans.fix(context=context)
        return ans

    def min(self, other, context=None):
        """Returns the smaller value.

        like min(self, other) except if one is not a number, returns
        NaN (and signals if one is sNaN).  Also rounds.
        """
        if context is None:
            context = getcontext()

        ans = self._check_nans(other, context)
        if ans: return ans


        ans = self

        if self > other:
            ans = other

        if context.rounding_decision in (ALWAYS_ROUND,):
            ans = ans.fix(context=context)

        return ans


    #def __coerce__(self, other):
    #    """Returns Decimal(other)"""
    #    if issubclass(other.__class__, self.__class__):
    #        return (self, other)
    #    try:
    #        context = getcontext()
    #        return (self, Decimal(other, context))
    #    except TypeError:
    #        return None

    def isinteger(self):
        """Returns whether self is an integer"""
        if self._exp >= 0:
            return Decimal(1)
        rest = self._int[self._exp:]
        if rest == (0,)*len(rest):
            return Decimal(1)
        return Decimal(0)

    def _iseven(self):
        """Returns 1 if self is even.  Assumes self is an integer."""
        if self._exp > 0:
            return 1
        return self._int[-1+self._exp] % 2 == 0

    def adjusted(self):
        """Return the adjusted exponent of self"""
        try:
            return self._exp + len(self._int) - 1
        #If NaN or Infinity, self._exp is string
        except TypeError: 
            return 0
    

class Context:
    """Contains the context for a Decimal instance.

    Contains:
    prec - precision (for use in rounding, division, square roots..)
    rounding - rounding type. (how you round)
    rounding_decision - ALWAYS_ROUND, NEVER_ROUND -- do you round?
    trap_enablers - If trap_enablers[exception] = 1, then the exception is
                    raised when it is caused.  Otherwise, a value is
                    substituted in.
    flags  - When an exception is caused, flags[exception] is incremented.
             (Whether or not the trap_enabler is set)
             Should be reset by user of Decimal instance.
    Emin -   Minimum exponent
    Emax -   Maximum exponent
    capitals -      If 1, 1*10^1 is printed as 1E+1.  If 0, printed as 1e1
    """
    def __init__(self, prec=None, rounding=None,
                 trap_enablers=None, flags=None,
                 rounding_decision=None,
                 Emin=None, Emax=None,
                 capitals=None, clamp=None):
        DefaultLock.acquire()
        for name, val in locals().items():
            if val is None:
                setattr(self, name, copy.copy(getattr(DefaultContext, name)))
            else:
                setattr(self, name, val)
        DefaultLock.release()
        del self.self

        self.ignored_flags = []

    def raise_error(self, error, explanation = None, *args):
        """Handles an error

        If the flag is in ignored_flags, returns the default response.
        Otherwise, it increments the flag, then, if the corresponding
        trap_enabler is set, it reaises the exception.  Otherwise, it returns
        the default value after incrementing the flag.
        """
        if error in self.ignored_flags:
            #Don't touch the flag
            return error().handle(self, *args) 
        
        self.flags[error] += 1
        if not self.trap_enablers[error]:
            #The errors define how to handle themselves.
            return error().handle(self, *args)
        
        # Errors should only be risked on copies of the context
        #self.ignored_flags = [] 
        raise error, explanation 
    
    def ignore_all_flags(self):
        """Ignore all flags, if they are raised"""
        return self.ignore_flags(*ExceptionList)

    def ignore_flags(self, *flags):
        """Ignore the flags, if they are raised"""
        # Do not mutate-- This way, copies of a context leave the original
        # alone.
        self.ignored_flags = (self.ignored_flags + list(flags)) 
        return list(flags)                              
                                                        

    def regard_flags(self, *flags):
        """Stop ignoring the flags, if they are raised"""
        if flags and type(flags[0]) in (type(()), type([])):
            flags = flags[0]
        for flag in flags:
            self.ignored_flags.remove(flag)

    def Etiny(self):
        """Returns Etiny (= Emin - prec + 1)"""
        return long(self.Emin - self.prec + 1)
    def Etop(self):
        """Returns maximum exponent (= Emin - prec + 1)"""
        return long(self.Emax - self.prec + 1)

    def all_flags(self):
        """Returns a list of all flags which have been tripped"""
        L = []
        for exception in ExceptionList:
            if self.flags[exception]:
                L.append(exception)
        return L
    def reset_flag(*flags):
        """Resets a flag"""
        for flag in flags:
            self.flags[flag] = 0
    def reset_all_flags():
        """Resets all flags"""
        self.reset_flag(*ExceptionList)
    
    
    def set_rounding_decision(self, type):
        """Sets the rounding decision.
        
        Sets the rounding decision, and returns the current (previous)
        rounding decision.  Often used like:

        context = copy.copy(context) 
        # That so you don't change the calling context
        # if an error occurs in the middle (say DivisionImpossible is raised).
                                     
        rounding = context.set_rounding_decision(NEVER_ROUND)
        instance = instance / Decimal(2)
        context.set_rounding_decision(rounding)
        
        This will make it not round for that operation.
        """
        
        rounding = self.rounding_decision
        self.rounding_decision = type
        return rounding

    def set_rounding(self, type):
        """Sets the rounding type.
        
        Sets the rounding type, and returns the current (previous)
        rounding type.  Often used like:

        context = copy.copy(context) 
        # so you don't change the calling context
        # if an error occurs in the middle.
        rounding = context.set_rounding(ROUND_UP)
        val = context.subtract(self, other)
        context.set_rounding(rounding)
        
        This will make it round up for that operation.
        """
        rounding = self.rounding
        self.rounding= type
        return rounding

    #Methods
    def abs(self, a):
        return a.__abs__(context=self)
    def add(self, a, b):
        return a.__add__(b, context=self)
    def apply(self, a):
        return str(a.fix(context=self))
    def compare(self, a, b):
        return a.__cmp__(b, context=self)
    def lt(self, a, b):
        return Decimal(int(self.compare(a, b)) == -1)
    def gt(self, a, b):
        return Decimal(int(self.compare(a, b)) == 1)
    def eq(self, a, b):
        return Decimal(int(self.compare(a, b)) == 0)
    def divide(self, a, b):
        return a.__div__(b, context=self)
    def divideint(self, a, b):
        return a.__floordiv__(b, context=self)
    def divmod(self, a, b):
        return a.__divmod__(b, context=self)
    def fix(self, a, prec=None):
        return a.fix(context=self, prec=prec)
    def integer(self, a):
        return a.round_to_integer(context=self)
    def max(self, a,b):
        return a.max(b, context=self)
    def min(self, a,b):
        return a.min(b, context=self)
    def minus(self, a):
        return a.__neg__(context=self)
    def multiply(self, a, b):
        return a.__mul__(b, context=self)
    def new(self, a):
        return Decimal(a, context=self)
    def normalize(self, a):
        return a.normalize(context=self)
    def plus(self, a):
        return a.__pos__(context=self)
    def power(self, a, b, modulo=None):
        return a.__pow__(b, modulo, context=self)
    def round(self, a, prec=None):
        return a.round(context=self, prec=prec)
    def remainder(self, a, b):
        return a.__mod__(b, context=self)
    def remaindernear(self, a, b):
        return a.remainder_near(b, context=self)
    def rescale(self, a, b):
        return a.rescale(b, context=self)
    def squareroot(self, a):
        return a.sqrt(context=self)
    def subtract(self, a, b):
        return a.__sub__(b, context=self)
    def toEng(self, a):
        return a.eng(context=self)
    def toSci(self, a):
        return a.sci(context=self)
    def trim(self, a):
        return a.trim(context=self)

class _WorkRep:
    def __init__(self, value=None):
        if value is None:
            self.sign = None
            self.int = []
            self.exp = None
        if isinstance(value, Decimal):
            if value._sign:
                self.sign = -1
            else:
                self.sign = 1
            self.int = list(value._int)
            self.exp = value._exp
        if isinstance(value, type( () )):
            self.sign = value[0]
            self.int = value[1]
            self.exp = value[2]

    def __repr__(self):
        return "(" + `self.sign` + ", " + `self.int` + ", " + `self.exp` + ")"

    __str__ = __repr__

    def __neg__(self):
        if self.sign == 1:
            return _WorkRep( (-1, self.int, self.exp) )
        else:
            return _WorkRep( (1, self.int, self.exp) )

    def __abs__(self):
        if self.sign == -1:
            return -self
        else:
            return self

    def __cmp__(self, other):
        if self.exp != other.exp:
            raise ValueError("Operands not normalized: " + `self` +
                             ", " + `other`)
        if self.sign != other.sign:
            if self.sign == -1:
                return -1
            else:
                return 1
        if self.sign == -1:
            direction = -1
        else:
            direction = 1
        int1 = self.int
        int2 = other.int
        if len(int1) > len(int2):
            return direction * 1
        if len(int1) < len(int2):
            return direction * -1
        for i in range(len(int1)):
            if int1[i] > int2[i]:
                return direction * 1
            if int1[i] < int2[i]:
                return direction * -1
        return 0

    def _increment(self):
        curspot = len(self.int) - 1
        self.int[curspot]+= 1
        while (self.int[curspot] >= 10):
            self.int[curspot] -= 10
            if curspot == 0:
                self.int[0:0] = [1]
                break
            self.int[curspot-1] += 1
            curspot -= 1

    def _subtract(self, list):
        """Subtract a list from the current int (in place).

        It is assured that (len(list) = len(self.int) and list < self.int) or
        len(list) = len(self.int)-1
        (i.e. that int(join(list)) < int(join(self.int)))
        """
        
        self.int.reverse()
        list.reverse()

        carry = 0
        for x in range(len(list)):
            self.int[x] -= list[x] + carry
            if self.int[x] < 0:
                carry = 1
                self.int[x] += 10
            else: carry = 0
        if carry:
            self.int[x+1] -= 1
        last = len(self.int)-1
        while len(self.int) > 1 and self.int[last] == 0:
            last -= 1
            if last == 0:
                break
        self.int[last+1:]=[]
        self.int.reverse()
        list.reverse()
        return
        


def _normalize(op1, op2, shouldround = 0, prec = 0):
    """Normalizes op1, op2 to have the same exp and length of coefficient.

    Done during addition.
    """
    # Yes, the exponent is a long, but the difference between exponents
    # must be an int-- otherwise you'd get a big memory problem.
    numdigits = int(op1.exp - op2.exp)
    if numdigits < 0:
        numdigits = -numdigits
        tmp = op2
        other = op1
    else:
        tmp = op1
        other = op2
    if shouldround and numdigits > len(other.int) + prec + 1 -len(tmp.int):
        # If the difference in adjusted exps is > prec+1, we know
        # other is insignificant, so might as well put a 1 after the precision.
        # (since this is only for addition.)  Also stops MemoryErrors.
        
        extend = prec + 2 -len(tmp.int)
        if extend <= 0:
            extend = 1
        tmp.int.extend([0]*extend)
        tmp.exp -= extend
        other.int[:] = [0]*(len(tmp.int)-1)+[1]
        other.exp = tmp.exp
        return op1, op2

    tmp.int.extend([0] * numdigits)
    tmp.exp = tmp.exp - numdigits
    numdigits = len(op1.int) - len(op2.int)
    # numdigits != 0 => They have the same exponent, but not the same length
    # of the coefficient.
    if numdigits < 0:                       
        numdigits = -numdigits
        tmp = op1
    else:
        tmp = op2
    tmp.int[0:0] = [0] * numdigits
    return op1, op2

def _adjust_coefficients(op1, op2):
    """Adjust op1, op2 so that op2.int+[0] > op1.int >= op2.int.

    Returns the adjusted op1, op2 as well as the change in op1.exp-op2.exp.
    
    Used on _WorkRep instances during division.
    """
    adjust = 0
    #If op1 is smaller, get it to same size
    if len(op2.int) > len(op1.int):  
        diff = len(op2.int) - len(op1.int)
        op1.int.extend([0]*diff)
        op1.exp -= diff
        adjust = diff

    #Same length, wrong order
    if len(op1.int) == len(op2.int) and op1.int < op2.int: 
        op1.int.append(0)
        op1.exp -= 1
        adjust+= 1
        return op1, op2, adjust 

    if len(op1.int) > len(op2.int) + 1:
        diff = len(op1.int) - len(op2.int) - 1
        op2.int.extend([0]*diff)
        op2.exp -= diff
        adjust -= diff

    if len(op1.int) == len(op2.int)+1 and op1.int > op2.int:

        op2.int.append(0)
        op2.exp -= 1
        adjust -= 1
    return op1, op2, adjust




BASIC_DEFAULT_TRAPS = {}
BASIC_DEFAULT_FLAGS = {}

EXTENDED_DEFAULT_TRAPS = {}
EXTENDED_DEFAULT_FLAGS = {}

for exception in ExceptionList:
    BASIC_DEFAULT_TRAPS[exception] = exception.default
    BASIC_DEFAULT_FLAGS[exception] = 0
    EXTENDED_DEFAULT_TRAPS[exception] = 0
    EXTENDED_DEFAULT_FLAGS[exception] = 0



DEFAULT_PRECISION = EXTENDED_DEFAULT_PRECISION
DEFAULT_ROUNDING = EXTENDED_DEFAULT_ROUNDING
DEFAULT_TRAPS = EXTENDED_DEFAULT_TRAPS
DEFAULT_FLAGS= EXTENDED_DEFAULT_FLAGS
DEFAULT_ROUNDING_DECISION = EXTENDED_DEFAULT_ROUNDING_DECISION

DefaultLock = threading.Lock()
DefaultContext = Context(DEFAULT_PRECISION, DEFAULT_ROUNDING,
                         DEFAULT_TRAPS.copy(),
                         DEFAULT_FLAGS.copy(), DEFAULT_ROUNDING_DECISION,
                         DEFAULT_MIN_EXPONENT, DEFAULT_MAX_EXPONENT,
                         CAPITALS, CLAMP)

#Used in SetDefaultContext
DefaultAttributeDict= {'DEFAULT_PRECISION' : 'prec',
                       'DEFAULT_TRAPS' : 'trap_enablers',
                       'DEFAULT_FLAGS' : 'flags',
                       'DEFAULT_ROUNDING' : 'rounding',
                       'DEFAULT_ROUNDING_DECISION' : 'rounding_decision',
                       'DEFAULT_MIN_EXPONENT' : 'Emin',
                       'DEFAULT_MAX_EXPONENT' : 'Emax'}


def SetDefaultContext(context=None):
    """Changes DefaultContext to the DEFAULT_*
    
    If context (default None) is BASIC_DEFAULT_CONTEXT or
    EXTENDED_DEFAULT_CONTEXT, sets DEFAULT_* to BASIC_DEFAULT_* or
    EXTENDED_DEFAULT_*.

    Whether that happened or not, change DefaultContext to use those.
    """
    global DEFAULT_CONTEXT, DEFAULT_PRECISION, DEFAULT_TRAPS
    global DEFAULT_FLAGS, DEFAULT_ROUNDING, DEFAULT_ROUNDING_DECISION
    global DEFAULT_MIN_EXPONENT, DEFAULT_MAX_EXPONENT
    DEFAULT_CONTEXT = context
    if DEFAULT_CONTEXT == BASIC_DEFAULT_CONTEXT:
        DEFAULT_PRECISION = BASIC_DEFAULT_PRECISION
        DEFAULT_ROUNDING = BASIC_DEFAULT_ROUNDING
        DEFAULT_TRAPS = BASIC_DEFAULT_TRAPS
        DEFAULT_FLAGS= BASIC_DEFAULT_FLAGS
        DEFAULT_ROUNDING_DECISION = EXTENDED_DEFAULT_ROUNDING_DECISION

    elif DEFAULT_CONTEXT == EXTENDED_DEFAULT_CONTEXT:
        DEFAULT_PRECISION = EXTENDED_DEFAULT_PRECISION
        DEFAULT_ROUNDING = EXTENDED_DEFAULT_ROUNDING
        DEFAULT_TRAPS = EXTENDED_DEFAULT_TRAPS
        DEFAULT_FLAGS= EXTENDED_DEFAULT_FLAGS
        DEFAULT_ROUNDING_DECISION = EXTENDED_DEFAULT_ROUNDING_DECISION

    for key, val in DefaultAttributeDict.items():
        setattr(DefaultContext, val,
                copy.copy(globals()[key]))



SetDefaultContext(DEFAULT_CONTEXT)

def isinfinity(num):
    """Determines whether a string or float is infinity."""
    try:
        if type(num) == type(1.0):
            num = str(num)
        num = num.lower()
        if num in ['inf', 'infinity', '+inf', '+infinity']:
            return 1
        if num in ['-inf', '-infinity']:
            return -1
    except AttributeError:
        pass
    return 0

def isnan( num):
    """Determines whether a string or float is NaN

    1 => NaN
    2 => sNaN
    """
    try:
        if type(num) == type(1.0):
            num = str(num)
        num = num.lower()
        if num in  ['nan']:
            return 1
        if num == 'snan':
            return 2
    except AttributeError:
        pass
    return 0

#Reusable defaults
Inf = Decimal('Inf')
negInf = Decimal('-Inf')

#Infsign[sign] is infinity w/ that sign
Infsign = (Inf, negInf)

NaN = Decimal('NaN')

# Code ripped more-or-less directly from FixedPoint.py
# _floatToString modified by Tim Peters to give exact results

# crud for parsing strings
import re

# There's an optional sign at the start, and an optional exponent
# at the end.  The exponent has an optional sign and at least one
# digit.  In between, must have either at least one digit followed
# by an optional fraction, or a decimal point followed by at least
# one digit.  Yuck.


_parser = re.compile(r"""
#    \s*
    (?P<sign>[-+])?
    (
        (?P<int>\d+) (\. (?P<frac>\d*))?
    |
        \. (?P<onlyfrac>\d+)
    )
    ([eE](?P<exp>[-+]? \d+))?
#    \s*
    $
""", re.VERBOSE).match #Uncomment the \s* to allow leading or trailing spaces.

del re

# return sign, n, p s.t. float string value == -1**sign * n * 10**p exactly

def _string2exact(s):
    m = _parser(s)
    if m is None:
        raise ValueError("Can't parse as number: " + `s`)

    if m.group('sign') == "-":
        sign = 1
    else:
        sign = 0

    exp = m.group('exp')
    if exp is None:
        exp = 0
    else:
        exp = long(exp)

    intpart = m.group('int')
    if intpart is None:
        intpart = ""
        fracpart = m.group('onlyfrac')
    else:
        fracpart = m.group('frac')
        if fracpart is None:
            fracpart = ""

    nfrac = len(fracpart)
    exp = exp - nfrac

    mantissa = intpart + fracpart
    tmp = []
    for digit in mantissa:
        tmp.append(int(digit))
    backup = tmp
    while tmp and tmp[0] == 0:
        del tmp[0]

    # It's a zero
    if not tmp:
        if backup:
            return (sign, tuple(backup), exp)
        return (sign, (0,), exp)
    mantissa = tuple(tmp)


    return (sign, mantissa, exp)



def _floatToString(x):
    """Return float x as exact decimal string.

    The string is of the form:
        "-", if and only if x is < 0.
        One or more decimal digits.  The last digit is not 0 unless x is 0.
        "e"
        The exponent, a (possibly signed) integer
    """

    import math

    if x == 0:
        return "0e0"

    sign = ""
    if x < 0:
        sign = "-"
        x = -x

    f, e = math.frexp(x)
    assert 0.5 <= f < 1.0
    # x = f * 2**e exactly

    # Suck up CHUNK bits at a time; 28 is enough so that we suck
    # up all bits in 2 iterations for all known binary double-
    # precision formats, and small enough to fit in an int.
    CHUNK = 28
    top = 0L
    # invariant: x = (top + f) * 2**e exactly
    while f:
        f = math.ldexp(f, CHUNK)
        digit = int(f)
        assert digit >> CHUNK == 0
        top = (top << CHUNK) | digit
        f = f - digit
        assert 0.0 <= f < 1.0
        e = e - CHUNK

    assert top > 0

    # Now x = top * 2**e exactly.  Get rid of trailing 0 bits if e < 0
    # (purely to increase efficiency a little later -- this loop can
    # be removed without changing the result).
    while e < 0 and top & 1 == 0:
        top = top >> 1
        e = e + 1

    # Transform this into an equal value top' * 10**e'.
    if e > 0:
        top = top << e
        e = 0
    elif e < 0:
        # Exact is top/2**-e.  Multiply top and bottom by 5**-e to
        # get top*5**-e/10**-e = top*5**-e * 10**e
        top = top * 5**-e

    # Nuke trailing (decimal) zeroes.
    while 1:
        assert top > 0
        newtop, rem = divmod(top, 10)
        if rem:
            break
        top = newtop
        e = e + 1

    return "%s%se%d" % (sign, str(top), e)

#Map the test cases' error names to the actual errors
ErrorNames = {'clamped' : Clamped,
              'conversion_syntax' : ConversionSyntax,
              'division_by_zero' : DivisionByZero,
              'division_impossible' : DivisionImpossible,
              'division_undefined' : DivisionUndefined,
              'inexact' : Inexact,
              'insufficient_storage' : InsufficientStorage,
              'invalid_context' : InvalidContext,
              'invalid_operation' : InvalidOperation,
              'lost_digits' : LostDigits,
              'overflow' : Overflow,
              'rounded' : Rounded,
              'subnormal' : Subnormal,
              'underflow' : Underflow}
              

def change_precision(precision):
    """Changes the precision of DefaultContext (used by new threads)."""
    global DEFAULT_PRECISION
    DEFAULT_PRECISION = precision
    SetDefaultContext()

def change_max_exponent(exp):
    """changes Emax used by new threads."""
    global DEFAULT_MAX_EXPONENT
    DEFAULT_MAX_EXPONENT = exp
    SetDefaultContext()

def change_min_exponent(exp):
    """changes Emin used by new threads"""
    global DEFAULT_MIN_EXPONENT
    DEFAULT_MIN_EXPONENT = exp
    SetDefaultContext()

def change_rounding_method(rounding):
    """changes the rounding method used by new threads."""
    global DEFAULT_ROUNDING
    DEFAULT_ROUNDING = rounding
    SetDefaultContext()

def change_rounding_decision(decision):
    """changes the decision of when to round used by new threads."""
    global DEFAULT_ROUNDING_DECISION
    DEFAULT_ROUNDING_DECISION = decision
    SetDefaultContext()


def Nonfunction(*args):
    """Doesn't do anything."""
    return None

RoundingDict = {'ceiling' : ROUND_CEILING, #Maps test-case names to roundings.
                'down' : ROUND_DOWN,
                'floor' : ROUND_FLOOR,
                'half_down' : ROUND_HALF_DOWN,
                'half_even' : ROUND_HALF_EVEN,
                'half_up' : ROUND_HALF_UP,
                'up' : ROUND_UP}

class Test:
    """Class which tests the Decimal class against the test cases.

    Example:
    t = Test('decimal/testcases/')
    t.test()

    The testcases should be unzipped in that directory (with names ending in
    .decTest)

    When run, test() stops for incorrect answers and gives a summary of
    results at the end.

    Changes DefaultContext to ensure no leaking (failure to pass the context)
    so watch out if you start a new thread!
    """
    def __init__(self, dir='./'):
        import glob
        self.filelist = glob.glob(dir+'*.decTest')
        if len(self.filelist) > 15:
            self.filelist.append(self.filelist[15])
            #So the slow one is at the end.
            self.filelist[15:16] = []

        self.context = Context()
        for key in DefaultContext.trap_enablers.keys():
            DefaultContext.trap_enablers[key] = 1

        self.unknown_list = ['#']
        # Basically, a # means return NaN InvalidOperation.
        # Different from a sNaN in trim

        self.ChangeDict = {'precision' : self.change_precision,
              'rounding' : self.change_rounding_method,
              'maxexponent' : self.change_max_exponent,
              'minexponent' : self.change_min_exponent,
              'clamp' : self.change_clamp}
  
    def test(self, start=0):
        wrong = 0
        notdone = 0
        total = 0
        t = time.time()
        for file in self.filelist[start:]:
            tup = self.eval_file(file)
            wrong += tup[0]
            notdone += tup[1]
            total += tup[2]

            #The clamp: directive only set in clamp cases, which leave it on.
            self.change_clamp(0)
        print 'The run-through took', time.time()-t, 'seconds.'
        print 'A total of', notdone, 'were skipped. (# ones)'
        print 'A total of', wrong, 'were incorrect.'
        print 'A total of', total-wrong, 'test cases were correct.'
    def eval_file(self, file):
        count = 0
        ndone = 0
        total = 0
        for line in open(file).xreadlines():
            line = line.replace('\r\n', '').replace('\n', '')
            #print '*', line, '*'
            try:
                t = self.eval_line(line)
            except ConversionSyntax:
                print 'PROBLEM WITH:'
                print line
                raw_input()
                continue
            count += t[0]
            ndone += t[1]
            total += t[2]
            #time.sleep(0.1)
        return count, ndone, total

    def eval_line(self, s):
        if s.find(' -> ') >= 0 and s[:2] != '--' and not s.startswith('  --'):
            s = (s.split('->')[0] + '->' +
                 s.split('->')[1].split('--')[0]).strip()
        else:
            s = s.split('--')[0].strip()
        
        for unknown in self.unknown_list:
            if s.find(unknown) >= 0:
                print s.split()[0], 'NotImplemented--', unknown
                return (0, 1, 0)
        if not s:
            return (0, 0, 0)
        if ':' in s:
            return self.eval_directive(s)
        else:
            return self.eval_equation(s)

    def eval_directive(self, s):
        funct, value = map(lambda x: x.strip().lower(), s.split(':'))
        if funct == 'rounding':
            value = RoundingDict[value]
        else:
            try:
                value = int(value)
            except ValueError:
                pass
        
        funct = self.ChangeDict.get(funct, Nonfunction)
        funct(value)
        return (0, 0, 0)

    def eval_equation(self, s):
        #global DEFAULT_PRECISION
        #print DEFAULT_PRECISION
        try:
            Sides = s.split('->')
            L = Sides[0].strip().split()
            id = L[0]
            print id,
            funct = L[1]
            valstemp = L[2:]
            L = Sides[1].strip().split()
            ans = L[0]
            exceptions = L[1:]
        except (TypeError, AttributeError, IndexError):
            raise ConversionSyntax
        def FixQuotes(val):
            val = val.replace("''", 'SingleQuote').replace('""', 'DoubleQuote')
            val = val.replace("'", '').replace('"', '')
            val = val.replace('SingleQuote', "'").replace('DoubleQuote', '"')
            return val
        fname = funct
        funct = getattr(self.context, funct)
        vals = []
        conglomerate = ''
        quote = 0
        theirexceptions = map(lambda x: ErrorNames[x.lower()], exceptions)

        for exception in ExceptionList:
            self.context.trap_enablers[exception] = 1 #Catch these bugs...
        for exception in theirexceptions:
            self.context.trap_enablers[exception] = 0
        for val in valstemp:
            if val.count("'") % 2 == 1:
                quote = 1 - quote
            if quote:
                conglomerate = conglomerate + ' ' + val
                continue
            else:
                val = conglomerate + val
                conglomerate = ''
            v = FixQuotes(val)
            if fname in ('toSci', 'toEng'):
                v = self.context.new(v)
            else:
                v = Decimal(v)
            vals.append(v)

        ans = FixQuotes(ans)

        try:
            result = str(funct(*vals))
        except: #Catch any error long enough to state the test case.
            print "ERROR:", s
            raise

        myexceptions = self.getexceptions()
        self.resetflags()

        myexceptions.sort()
        theirexceptions.sort()
        #if s.count('pow') and result != ans:
        #    #Maybe just the one digit..
        #    spot = result.find('E')
        #    if abs(long(result[:spot].replace('.', '')) -
        #           long(ans[:spot].replace('.', ''))) == 1:
        #           
        #        result = ans[:spot] + result[spot:]
        #        print 'OFF BY ONE IN LAST DIGIT OTHERWISE', 
                
        if result != ans or myexceptions != theirexceptions:
            print 'INCORRECT **********'
            print 'PROBLEM:'
            print s
            print 'YOUR RESULT:'
            print result, ' '.join(map(lambda x: x.__name__, myexceptions))
            raw_input()
            return (1, 0, 1)
        else:
            print 'CORRECT!'
        return (0, 0, 1)

    def getexceptions(self):
        return self.context.all_flags()
        L = []
        for exception in ExceptionList:
            if self.context.flags[exception]:
                L.append(exception)
        return L

    def resetflags(self):
        for exception in ExceptionList:
            self.context.flags[exception] = 0

    def change_precision(self, prec):
        self.context.prec = prec
    def change_rounding_method(self, rounding):
        self.context.rounding = rounding
    def change_min_exponent(self, exp):
        self.context.Emin = exp
    def change_max_exponent(self, exp):
        self.context.Emax = exp
    def change_clamp(self, clamp):
        self.context.clamp = clamp

            
        

def _test():
    t = Test('./')
    t.test()

if __name__ == '__main__':
    _test()
